import Test.QuickCheck
import Data.List

-- Definición de función:
qsort :: Ord a => [a] -> [a]
qsort []     = []
qsort (x:xs) = qsort lhs ++ [x] ++ qsort rhs
    where lhs = filter  (< x) xs
          rhs = filter (>= x) xs

-- Propiedades que vamos a testear:

-- Idempotencia:
prop_idempotent xs = qsort (qsort xs) == qsort xs

-- El primer elemento es el mínimo:
prop_minimum xs = not (null xs) ==> head (qsort xs) == minimum xs

-- El último elemento es el máximo:
prop_maximum xs = not (null xs) ==> last (qsort xs) == maximum xs

-- La lista está ordenada:
prop_ordered xs = ordered (qsort xs)
    where ordered []       = True
          ordered [x]      = True
          ordered (x:y:xs) = x <= y && ordered (y:xs)

-- La lista ordenada es permutación de la lista original:
prop_permutation xs = permutation xs (qsort xs)
    where permutation xs ys = null (xs \\ ys) && null (ys \\ xs)

-- Comparar con un sort que sabemos que anda:
prop_sort_model xs = sort xs == qsort xs
