import Test.QuickCheck
import Control.Monad

-- Declaración del Tipo de Dato:

data Arbol a = Hoja | Rama (Arbol a) a (Arbol a) deriving (Show, Eq)

aFold :: (b -> a -> b -> b) -> b -> (Arbol a) -> b
aFold f z Hoja = z
aFold f z (Rama ai a ad) = f (aFold f z ai) a (aFold f z ad)

-- Propiedades que vamos a testear:

prop_neutro arbol = not ((aFold Rama Hoja arbol) == arbol)


-- Forma Simple:
instance Arbitrary a => Arbitrary (Arbol a) where
    arbitrary = do
        n <- choose (1,2) :: (Gen Int)
        case n of
            1 -> return Hoja
            2 -> do ai <- arbitrary
                    v <- arbitrary
                    ad <- arbitrary
                    return (Rama ai v ad)

-- Forma más elaborada:
{-
instance Arbitrary a => Arbitrary (Arbol a) where
    arbitrary = sized arbol_gen
        where
            arbol_gen 0 = return Hoja
            arbol_gen n | n>0 = frequency [(1,return Hoja), (3,liftM3 Rama subtree arbitrary subtree)]
                where subtree = arbol_gen (n `div` 2)
-}

data Color = Rojo | Verde | Azul | Amarillo deriving (Eq, Show)

instance Arbitrary Color where
    arbitrary = elements [Rojo, Verde, Azul, Amarillo]

{- instance Arbitrary Color where
    arbitrary = do
        n <- choose (1,4) :: (Gen Int)
        case n of
            1 -> return Rojo
            2 -> return Verde
            3 -> return Azul
            4 -> return Amarillo
-}
